#!/usr/local/bin/perl -w # [/usr/local/bin/perl]
###########################################################################
###########################################################################
#   
#	   Perl script for getting patches for Sun Solaris
#
#		 Copyright (c) 1999 by Roman M Ivanov
#   
#		  Current version: v1.5, 21 May 2001
#
#               Modified and improved by Mick Ohrberg   
#
###########################################################################
#
#   Aim: 
#     get_patches is designed to download patches from Sun's Sunsolve site,
#     according to your system's requrements in the cross-reference file.
#     It's written in a way that it can be automated (with a lot of 
#     commandline options), or interactive (ask for usernames and pass-
#     words). 
#   
#   How it works: 
#     It connects to Sunsolve site, retrieves the cross-reference list,
#     analyzes it with the patchdiag tool, then retrieves all you need to
#     fulfill the cross-reference list. For patches unlisted in the cross-
#     reference file, use the -patch option.
#   
#   What it needs: 
#     * PERL (5.005_03 or later)
#       - LWP module (prerequisites: Digest-MD5, libnet, HTML::Parse,
#                     MIME::Base64, HTML::Tagset, URI)
#       - Term::ReadKey
#     * PATCHDIAG
# (http://sunsolve.Sun.COM/private-cgi/show.pl?target=resources/patchdiag,
#       requires SunSolve login)
#
#   Future:
#     Something I'd like to do is write a Perl/Tk GUI for it. If anyone
#     has any experience with Perl/Tk and would like to contribute, don't
#     hesitate to get with me.
#
###########################################################################
#
#   Bugs and improvements: I'm very thankful if you inform me about
#   bugs you find and improvements you made. You may contact me at
#   micke@kc.rr.com
#   
###########################################################################

use Getopt::Long;
use LWP;
use LWP::UserAgent;
use Term::ReadKey;

###########################################################################
###########################################################################
#   
# THIS SHOULD BE EDITED, but it is not necessary, since options can be set 
# from the command line.
#
# NOTE That the user_proxy is set to 0 by default. You will have to specify
# -proxy on the command line to utilize a proxy unless you modify the 
# default value below. -- $use_proxy = 1;
#
# NOTE Also that wget is no longer used! Instead I have implemented the
# slicker LWP module. In my experience, if you do:
# 
#     # perl -MCPAN -e shell
#     cpan> install Bundle::CPAN
#
# you will get all the modules installed as part of the CPAN upgrade 
# procedure. A WORD OF WARNING. If you have an old(er) version of PERL,
# the above process tends to attempt to upgrade your PERL as well.
#
###########################################################################
###########################################################################

$server                = 'sunsolve.sun.com'; # [sunsolve.sun.com]
$use_proxy             = 0; # Default is to not use a proxy
$patch_dir             = '/opt/patch';     # [/opt/patch]
$patchdiag_dir         = '/opt/patch/patchdiag-latest'; # [/opt/patch/patchdiag-latest]
$max_files_to_transfer = 15;
###########################################################################
# All the ones below can be predefined. If they are not, the user will
# be prompted for them.
###########################################################################
$user                  = 'tpsamo'; #<------ MODIFY THIS!!!
$passwd                = ''; #<------ MODIFY THIS!!!
$proxy_user            = 'mohrberg'; #<------ MODIFY THIS!!!
$proxy_passwd          = ''; #<------ MODIFY THIS!!!
$proxy_server          = 'umbproxy.umb.com'; #<------ MODIFY THIS!!!
$proxy_port            = 9119; #<------ MODIFY THIS!!!

###########################################################################

###########################################################################
# Setting some initial values
###########################################################################
$verbose               = 0; # 0 - silent, 1 - verbose, 2 - more verbose
undef                    $separate_patches;
$noxref                = 0;

###########################################################################
# NOT RECOMMENDED TO CHANGE - EXPERIMENTAL!
###########################################################################
$ignore_if_have        = 1;
$max_files_to_delete   = 2;

###########################################################################
# DO NOT ENABLE THIS! IT DOES NOT WORK YET!
###########################################################################
$remove_expired        = 0; # 1 - Yes
$transferred_files     = 0;
$|                     = 1;
select                   STDOUT;

###########################################################################
# First, let's do some command line option parsing. These options over-
# ride the hardcoded stuff.
###########################################################################

if ( GetOptions(
		"help|h"           => \$help,
		"proxy"            => \$use_proxy,
		"noxref"           => \$noxref,
		"server=s"         => \$server, 
		"user=s"           => \$user, 
		"password=s"       => \$passwd, 
		"proxy_user=s"     => \$proxy_user, 
		"proxy_passwd=s"   => \$proxy_passwd,
		"proxy_server=s"   => \$proxy_server,
		"proxy_port=i"     => \$proxy_port,
		"out_dir=s"        => \$patch_dir,
		"patchdiag_dir=s"  => \$patchdiag_dir,
		"max=i"            => \$max_files_to_transfer,
		"patch=s"          => \$separate_patches
		) == 0 ) { help(); exit(1); }
		  
if ($help) { help(); exit(0); }


###########################################################################
# Now we need to validate some of the option parameters. Also, if no user-
# names and passwords are given, we have to ask for them!
###########################################################################
if ( $use_proxy && ( $proxy_server eq '' ) ) {
    print "Please specify proxy server: ";
    $proxy_server = <>;
    chomp $proxy_server;
}

if ( $use_proxy && ( $proxy_port == 0 ) ) {
    print "Please specify proxy server port: ";
    $proxy_port = <>;
    chomp $proxy_port;
}

if ( $user eq '' ) {
    print "What is your SunSolve login name? ";
    $user = <>;
    chomp $user;
}

if ( $passwd eq '' ) {
    print "What is the password for SunsSolve user $user? ";
    ReadMode("noecho",STDIN);
    $passwd = <>;
    ReadMode("original",STDIN);
    print "\n";
    chomp $passwd;
}

if ( $use_proxy & ( $proxy_user eq '' ) ) { 
    print "What is your proxy username? ";
    $proxy_user = <>;
    chomp $proxy_user;
}

if ( $use_proxy & ( $proxy_passwd eq '' ) ) { 
    print "What is the proxy password for $proxy_user? ";
    ReadMode("noecho",STDIN);
    $proxy_passwd = <>;
    ReadMode("original",STDIN);
    print "\n";
    chomp $proxy_passwd;
}

if ( $max_files_to_transfer < 1 ) {
    print "A max-value of $max_files_to_transfer makes no sense. Using default value of 15.\n";
    $max_files_to_transfer = 15;
}

if ( ! -e $patch_dir ) { 
    print "Option out_dir requires an existing path.\n$patch_dir does not exist.\n";
    help();
    exit(1);
}
if ( ! -x "$patchdiag_dir/patchdiag" ) { 
    print "Option patchdiag_dir requires a path to an existing and executable patchdiag\nscript. $patchdiag_dir does not exist, or $patchdiag_dir/patchdiag is not\nexecutable.\n";
    help();
    exit(1);
}
if ( $separate_patches ) {
    if ( $separate_patches =~ "tar.Z" ) {
	print "Please specify patches without suffix tar.Z\n";
	help();
	exit(1);
    }
}

if ( $use_proxy == 1 ) {
    $full_proxy_addy = "http://$proxy_server:$proxy_port/";
    print("Connecting to http://$server/, using $full_proxy_addy\n");
}

###########################################################################
# Here's what we need to do if -patch was given, meaning the user wants to
# download specific patches not listed in the xref-file!
###########################################################################
if ($separate_patches) { get_separate_patch(); exit(0) };

###########################################################################
# Here we check if the user really wants to download the cross-reference
# file again. If this is the second time around this utility is used on 
# the same day and machine, it's very unlikely the cross-reference file
# has changed, and therefore we can save some time here.
###########################################################################
if (! $noxref ) {
###########################################################################
# Set up the LWP UserAgent that we will be sending all the http requests to
###########################################################################
    $ua = LWP::UserAgent->new;
    $ua->agent("Mozilla_Schmozilla/0.00a " . $ua->agent);
    $stuff="http://$user:$passwd\@$server/private-cgi/patches/patchdiag.xref";
    $req = HTTP::Request->new(GET => $stuff);
    if($use_proxy == 1) {
	$ua->proxy('http' => $full_proxy_addy);
	$req->proxy_authorization_basic($proxy_user, $proxy_passwd);
    } else {
	$ua->no_proxy();
    }
    printf("Getting patchdiag.xref -- ");
    my $local_file="$patch_dir/patchdiag.xref";
    my $res = $ua->request($req,$local_file);
# Check the outcome of the request
    if ($res->is_success) {
	my $cont = $res->content;
	printf("%s\n",$res->status_line);
    } else {
	printf("%s\n",$res->status_line);
	print "Bad luck this time.\n";
	exit(1);
    }

###########################################################################
# Here we analyze the cross-reference file and compare it to installed
# patches.
###########################################################################
    `$patchdiag_dir/patchdiag -x $patch_dir/patchdiag.xref >$patch_dir/patchdiag.log`;
#print "See $patch_dir/patchdiag.log\n";
}

###########################################################################
# get patches
###########################################################################
#print "Getting patches\n";
open (PL, "$patch_dir/patchdiag.log") || die "Error opening patchdiag.log\n";

###########################################################################
# list what I already have

@have = split('\n', `ls $patch_dir`);
if ($verbose > 0) {
    foreach $name (@have) {
    }
}
while (<PL>) {
    chop;
    if ($verbose > 1) {
    }
    if ( /^([0-9]{1,})([ ]{1,})([^ ]{1,})([ ]{1,})([^ ]{1,})(.{1,})/ ) {
	$patch_id = $1;
	$latest_rev = $5;
	$descr = $6;
	$obsoleted = 0;
	study $descr;
	if (/OBSOLETED/i) { $obsoleted = 1; }
	if (($latest_rev ne "CURRENT") and $obsoleted == 0) {
	    push @patches, "$patch_id-$latest_rev";
	    $sg = 1;
# may be i have this file?
	    foreach $name (@have) {
		if ($ignore_if_have == 1 and $name eq "$patch_id-$latest_rev.tar.Z") { $sg = 0; }
	    }
	    if ($sg == 1) {
		if ($transferred_files < $max_files_to_transfer) {
# make list of files to download
		    if (not defined $plist) { $plist = "$patch_id-$latest_rev"; }
		    else { $plist .= ",$patch_id-$latest_rev"; }
		    $transferred_files++;
		}
	    } else {
	    }
	} else {
	    if ($latest_rev eq "CURRENT") {
	    }
	}
    }
}

###########################################################################
#print "Doing real wget now...\n";
###########################################################################
if (defined $plist) {
    @get_patches = split(/,/,$plist);
    $array_size = @get_patches;
    while ($array_size > 0) {
	chomp $get_patches[$array_size-1];
	$ua = LWP::UserAgent->new;
	$ua->agent("Mozilla_Schmozilla/0.00a " . $ua->agent);
	$stuff="http://$user:$passwd\@$server/private-cgi/patchDownload.pl?target=$get_patches[$array_size-1]&method=h";
	$req = HTTP::Request->new(GET => $stuff);
	if ($use_proxy == 1) {
	    $ua->proxy('http' => $full_proxy_addy);
	    $req->proxy_authorization_basic($proxy_user, $proxy_passwd);
	} else {    
	    $ua->no_proxy();
	}
	print("Getting $get_patches[$array_size-1] -- ");
	my $local_file="$patch_dir/$get_patches[$array_size-1].tar.Z";
	my $res = $ua->request($req,$local_file);
# Check the outcome of the response
	if ($res->is_success) {
	    printf("%s\n",$res->status_line);
	} else {
	    printf("%s\n",$res->status_line);
	    print "Bad luck this time\n";
	    exit(1);
	}
	$array_size--;
    }
} else {
    print "No patches to transfer\n";
}
$deleted_files = 0;

###########################################################################
# remove old releases
###########################################################################
if ($remove_expired == 1) {
    foreach $name (@have) {
	$delete = 1;
	foreach $patch (@patches) {
	    if ($name eq "$patch.tar.Z") {
		$delete = 0;
	    }
	}
	if ($delete == 1) {
	    print "Should delete: $name\n";
	    if ($deleted_files < $max_files_to_delete) {
		`rm patches/$name`;
		$deleted_files++;
	    }
	}
    }
} else { print "Removing disabled\n"; }
close PL;
exit(0);


###########################################################################
###########################################################################
#
# END OF MAIN SECTION
#
# Now for some sub-routines
#
###########################################################################
###########################################################################


###########################################################################
sub help {
###########################################################################
    print "\nUsage: get_patches.pl [ -proxy ]\n";
    print "                      [ -noxref ]\n";
    print "                      [ -proxy_user username ]\n";
    print "                      [ -proxy_passwd password ]\n";
    print "                      [ -server server ]\n";
    print "                      [ -user username]\n";
    print "                      [ -proxy_server url ]\n";
    print "                      [ -proxy_port # ]\n";
    print "                      [ -password password]\n";
    print "                      [ -out_dir path ]\n";
    print "                      [ -patchdiag_dir path ]\n";
    print "                      [ -max number ]\n";
    print "                      [ -patch patch-patchrev,patch-patchrev, ... ]\n\n";
    print "All these options (except -patch) are defined in the script, but command-line\noptions will override the scripted defaults.\n\n";
    print "The -proxy switch specifies if you go through a proxy or not. The address of\nthe proxy needs to be specified in your ~/.wgetrc.\n";
    }


###########################################################################
sub get_separate_patch {
###########################################################################
    @separate_patches_array = split(',',$separate_patches);
    foreach $separate_patch (@separate_patches_array){
	$ua = LWP::UserAgent->new;
	$ua->agent("Mozilla_Schmozilla/0.00a " . $ua->agent);
	$stuff="http://$user:$passwd\@$server/private-cgi/patchDownload.pl?target=$separate_patch*&method=h";
	$req = HTTP::Request->new(GET => $stuff);
	if ($use_proxy == 1) {
	    $ua->proxy('http' => $full_proxy_addy);
	    $req->proxy_authorization_basic($proxy_user, $proxy_passwd);
	} else {
	    $ua->no_proxy();
	}
	print("Getting $separate_patch -- ");
	my $local_file="$patch_dir/$separate_patch.tar.Z";
	my $res = $ua->request($req,$local_file);
# Check the outcome of the response
	if ($res->is_success) {
	    printf("%s\n",$res->status_line);
	} else {
	    printf("%s\n",$res->status_line);
	    print "Bad luck this time\n";
	    exit(1);
	}

    }
}
